/**
 * Project: Max'Com: an oral communication software solution that provides
 * pre-recorded sound <-> daylife image / word associations. Autistic people
 * or persons with communication problems are to be the main users. The
 * content is sorted by categories that helps users to express themselves
 * according to the context.
 * File name: ConfigurationData.java
 * Description: this class is used to save and modify the configuration.
 * 
 * @author Telecom Lille1, ISEN Lille on 2010-11-23
 * @copyright 2010 Max'Com team. All rights reserved.
 * @version $Id: code-gpl-license.txt,v 1.2 2004/05/04 13:19:30 garry Exp $
 * 
 * @see The GNU Public License (GPL) version 3 or higher
 *
 * 
 * Max'Com is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package maxcom.mobile;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import android.content.Context;
import android.net.Uri;
import android.util.Log;
import android.widget.Toast;

import java.util.Properties;

public class ConfigurationData {

	public static final String tag = ConfigurationData.class.getName();
	
	private static final String CONFIG_FILE = "maxcom.cfg";
	protected static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
	
	protected static Uri serverUri;
	protected static String login;
	protected static String loginPassword;
	protected static String appPassword;
	protected static Date lastUpdate; 
	protected static boolean restartApp = false;
	
	
	protected static boolean isLoaded = false;
	protected static boolean isModified = false;
	
	/**
	 * Indique si la configuration est chargee
	 * @return l'tat du chargement
	 */
	public static boolean isLoaded() {
		return isLoaded;
	}

	/**
	 * Indique si la configuration est modifiee depuis le dernier chargement
	 * @return l'tat de modification
	 */
	public static boolean isModified() {
		return isModified;
	}	
	
	/**
	 * Ecriture dans le fichier
	 * @param context
	 */
	public static void save(Context context) {
		FileOutputStream fOut = null;
		OutputStreamWriter osw = null;
		try {
			fOut = context.openFileOutput(CONFIG_FILE, Context.MODE_PRIVATE);

			Properties properties = new Properties();
			properties.put("serverUri", serverUri.toString());
			properties.put("login", login);
			properties.put("loginPassword", loginPassword);
			properties.put("appPassword", appPassword);
			properties.put("restartApp", Boolean.toString(restartApp));		
			properties.put("lastUpdate", dateFormat.format(lastUpdate));
			properties.store(fOut, "maxcom mobile configuration file");
			
			isModified = false; 
			fOut.flush();
		} catch (Exception e) {
			if (BuildConfig.DEBUG)
				Log.e(tag, e.toString());
			Toast.makeText(context, "Erreur d'ecriture", Toast.LENGTH_SHORT)
					.show();
		} finally {
			try {
				fOut.close();
			} catch (IOException e) {
				if (BuildConfig.DEBUG) Log.e(tag, e.toString());
				Toast.makeText(context, "Erreur de fermeture",
						Toast.LENGTH_SHORT).show();
			}
		}
	} 

	/**
	 * Lecture du fichier
	 * @param context
	 * @return String data
	 */
	public static void load(Context context) {
		FileInputStream fIn = null;
		InputStreamReader isr = null;
		Properties properties = new Properties();
		
		try {
			fIn = context.openFileInput(CONFIG_FILE);
			properties.load(fIn);
			fIn.close();
			
			serverUri = Uri.parse(properties.getProperty("serverUri"));
			login = properties.getProperty("login");
			loginPassword = properties.getProperty("loginPassword");
			appPassword = properties.getProperty("appPassword");
			Boolean restart = Boolean.parseBoolean(properties.getProperty("restartApp")); 
			restartApp = restart.booleanValue();
			String dateStr = properties.getProperty("lastUpdate");
			lastUpdate = dateFormat.parse(dateStr);
			isLoaded = true; 
		} 
		catch (ParseException e) {
			restartApp = false;
		}
		catch (Exception e) {
			Toast.makeText(context, "Erreur en lecture", Toast.LENGTH_SHORT)
					.show();
		}
	}
	
	/**
	 * Retourne le serveur 
	 * @return serveur de l'appli
	 */
	public static Uri getServerUri() {
		return serverUri;
	}


	/**
	 * Retourne le login 
	 * @return login de l'appli
	 */
	public static String getLogin() {
		return login;
	}
	
	/**
	 * Retourne le mot de passe en clair associe au login
	 * @return mot de pass de l'appli
	 */
	public static String getLoginPassword(){
		return loginPassword; 
	} 
	
	
	/**
	 * Retourne le mot de passe en clair de l'appli 
	 * @return mot de pass de l'appli
	 */
	public static String getAppPassword(){
		return appPassword; 
	} 
	
	
	/**
	 * Retourne le mot de passe en MD5 associe au login 
	 * @return mot de pass de l'appli
	 */
	public static String getEncryptedLoginPassword(){
		return md5(loginPassword); 
	} 
	
	
	/**
	 * Retourne le mot de passe en clair de l'appli
	 * @return mot de pass de l'appli
	 */
	public static String getEncryptedAppPassword(){
		return md5(appPassword); 
	} 
	
	/**
	 * Retourne la date de derniere mise a jour
	 * @return date de derniere mise a jour
	 */
	public static Date getDateLastUpdate(){
		return lastUpdate;
	}
	
	/**
	 * Retourne l'tat du redmarrage automatique
	 * @return redmarrage automatique
	 */
	synchronized public static boolean mustRestartApp() {
		return restartApp;
	}
	
	/**
	 * Change l'tat du redmarrage automatique
	 */
	
	synchronized public static void switchRestartApp() {
		restartApp = !restartApp;
	}
	
	
	
	/**
	 * Modifie le serveur 
	 * @param Url serveur a memoriser
	 */
	public static void setServerUri(String uri) {
		serverUri = Uri.parse(uri);
		isModified = true;
	}
	
	/**
	 * Modifie le login
	 * @param login a memoriser
	 */
	public static void setLogin(String l) {
		login = l;
		isModified = true; 
	}

	/**
	 * Modifie le mot de passe associe au login
	 * @param mot de passe en clair a memoriser en md5
	 */
	public static void setLoginPassword(String pwd) {
		loginPassword = pwd;
		isModified = true;
	}

	/**
	 * Modifie le mot de passe associe a l'appli
	 * @param mot de passe en clair a memoriser en md5
	 */
	public static void setAppPassword(String pwd) {
		appPassword = pwd;
		isModified = true;
	}
	
	/**
	 * Indique si les mots de passe appli correspondent
	 * @param mot de passe en clair 
	 * @result resultat de la correspondance
	 */
	public static boolean matchAppPassword(String pwd) {
		return appPassword.equals(pwd);
	}

	
	/**
	 * Modifie la date de derniere mise a jour
	 * @param date  mmoriser
	 */
	public static void setLastUpdate(Date date) {
		lastUpdate = date;
		isModified = true;
	}
	
	/**
	 * Reinitialise la date de dernire mise  jour
	 */
	public static void resetUpdateDate() {
		lastUpdate = new Date(100, 0, 1);
		isModified = true;
	}
	

	/**
	 * Test si la configuration existe
	 * @param context
	 * @return existence ou non
	 */
	public static boolean isConfigCreated(Context context) {
		try {
			FileInputStream fIn = context.openFileInput(CONFIG_FILE);	
			fIn.close();
			return true;
		} catch (Exception e){
			return false;
		}
	}

	/**
	 * Transforme une chaine en md5
	 * @param String s
	 * @return String md5
	 */
	protected static String md5(String s) {
		try {
			MessageDigest digest = java.security.MessageDigest.getInstance("MD5");
			digest.update(s.getBytes());
			byte messageDigest[] = digest.digest();		
			// Create Hex String
	        StringBuffer hexString = new StringBuffer();
	        for (int i = 0; i < messageDigest.length; i++) {
	            String h = Integer.toHexString(0xFF & messageDigest[i]);
	            while (h.length() < 2)
	                h = "0" + h;
	            hexString.append(h);
	        }
	        return hexString.toString();
		} catch (NoSuchAlgorithmException e) {
			if (BuildConfig.DEBUG)
				Log.e(tag, e.toString());
		}
		return "";
	}

	
  	
}
