/**
 * Project: Max'Com: an oral communication software solution that provides
 * pre-recorded sound <-> daylife image / word associations. Autistic people
 * or persons with communication problems are to be the main users. The
 * content is sorted by categories that helps users to express themselves
 * according to the context.
 * File name: DatabaseHelper.java
 * Description: this class is used to create the database structure.
 * 
 * @author Telecom Lille1 on 2010-11-23
 * @copyright 2010 Max'Com team. All rights reserved.
 * @version $Id: code-gpl-license.txt,v 1.2 2004/05/04 13:19:30 garry Exp $
 * 
 * @see The GNU Public License (GPL) version 3 or higher
 *
 * 
 * Max'Com is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package maxcom.mobile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class DatabaseHelper extends SQLiteOpenHelper {

	public static final String tag = DatabaseHelper.class.getName();

	protected static final int VERSION_BDD = 8;
	protected static final String NOM_BDD = "maxcom.db";

	protected static final String ALL_ROWS = "1";

	public static final String CATEGORY_TABLE = "category";
	public static final String SHEET_TABLE = "sheet";
	public static final String TAXONOMY_TABLE = "taxonomy";

	protected static final int NUM_TAX_CAT = 0;
	protected static final int NUM_TAX_TERM = 1;
	protected static final int NUM_TAX_SHEET = 2;

	public static final String TERM_TABLE = "term";

	protected static final int NUM_TERM_ID = 0;
	protected static final int NUM_TERM_SPACE_ID = 1;
	protected static final int NUM_TERM_NAME = 2;

	protected static final String CREATE_BDD_CATEGORY = "CREATE TABLE "
			+ CATEGORY_TABLE + " (" + DataFieldDictionary.CAT_ID
			+ " INTEGER PRIMARY KEY, "
			+ DataFieldDictionary.CAT_NAME + " TEXT NOT NULL, "
			+ DataFieldDictionary.CAT_IMAGE + " TEXT NOT NULL, "
			+ DataFieldDictionary.CAT_PARENT + " INTEGER, "
			+ DataFieldDictionary.CAT_RANK + " INTEGER);";
	
	protected static final String CREATE_BDD_SHEET = "CREATE TABLE "
			+ SHEET_TABLE + " (" + DataFieldDictionary.SHEET_ID
			+ " INTEGER PRIMARY KEY, " + DataFieldDictionary.SHEET_NAME
			+ " TEXT NOT NULL, " + DataFieldDictionary.SHEET_IMAGE
			+ " TEXT NOT NULL, " + DataFieldDictionary.SHEET_FAVORITE
			+ " INTEGER, " + DataFieldDictionary.SHEET_STATUS + " INTEGER, "
			+ DataFieldDictionary.SHEET_NBCLICKS + " INTEGER, "
			+ DataFieldDictionary.SHEET_NBCLICKS_DELTA + " INTEGER, "
			+ DataFieldDictionary.SHEET_DATE + " TEXT, "
			+ DataFieldDictionary.SHEET_LAST_MODIF + " TEXT, "
			+ DataFieldDictionary.SHEET_CATEGORY_ID + " INTEGER, "
			+ DataFieldDictionary.SHEET_AUTHOR_ID + " INTEGER, "
			+ DataFieldDictionary.SHEET_NOTE + " INTEGER);";

	protected static final String CREATE_BDD_TAXO = "CREATE TABLE "
			+ TAXONOMY_TABLE + " (" 
			+ " INTEGER, " + DataFieldDictionary.TAX_TERM + " INTEGER, "
			+ DataFieldDictionary.TAX_SHEET + " INTEGER);";

	protected static final String CREATE_BDD_TERM = "CREATE TABLE "
			+ TERM_TABLE + " (" + DataFieldDictionary.TERM_ID
			+ " INTEGER PRIMARY KEY, "
			+ DataFieldDictionary.TERM_SPACE_ID + " INTEGER, "
			+ DataFieldDictionary.TERM_NAME + " TEXT);";

	protected SQLiteDatabase mDatabase;

	protected List<Category> categories;
	protected List<Sheet> sheets;
	protected List<Term> terms;
	protected List<Taxonomy> taxonomies;

	protected boolean needRefresh;

	public DatabaseHelper(Context context) {
		super(context, NOM_BDD, null, VERSION_BDD);
		categories = new LinkedList<Category>();
		sheets = new LinkedList<Sheet>();
		terms = new LinkedList<Term>();
		taxonomies = new LinkedList<Taxonomy>();

		needRefresh = true;
	}

	/**
	 * Fonction appelee au premier lancement
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		// on créé les tables à partir des requêtes écrites dans les
		// variable CREATE_BDD_...
		db.execSQL(CREATE_BDD_SHEET);
		db.execSQL(CREATE_BDD_CATEGORY);
		db.execSQL(CREATE_BDD_TAXO);
		db.execSQL(CREATE_BDD_TERM);

	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		db.execSQL("DROP TABLE IF EXISTS " + CATEGORY_TABLE);
		db.execSQL("DROP TABLE IF EXISTS " + SHEET_TABLE);
		db.execSQL("DROP TABLE IF EXISTS " + TAXONOMY_TABLE);
		db.execSQL("DROP TABLE IF EXISTS " + TERM_TABLE);
		onCreate(db);
	}

	/**
	 * Open the database.
	 */
	public void open() throws SQLException {
		mDatabase = this.getWritableDatabase();
		mDatabase.setLocale(Locale.FRENCH);
	}

	/**
	 * On ferme l'acces a la BDD
	 */
	@Override
	public void close() {
		mDatabase.close();
	}
	
	public boolean isOpened() {
		return mDatabase != null && mDatabase.isOpen();
	}

	public SQLiteDatabase getDatabase() {
		return mDatabase;
	}

	/**
	 * Insertion d'une categorie dans la BDD. Utilisation d'un ContentValues
	 * (fonctionne comme une HashMap)
	 * 
	 * @param category
	 * @return code d'erreur
	 */
	public long insertCategory(Category category) {
		ContentValues values = new ContentValues();
		
		values.put(DataFieldDictionary.CAT_ID, category.getId());
		values.put(DataFieldDictionary.CAT_NAME, category.getName());
		values.put(DataFieldDictionary.CAT_IMAGE, category.getImage());
		values.put(DataFieldDictionary.CAT_PARENT, category.getParentId());
		values.put(DataFieldDictionary.CAT_RANK, category.getRank());
		return mDatabase.insert(CATEGORY_TABLE, null, values);
	}

	/**
	 * Insertion d'une fiche dans la BDD. Utilisation d'un ContentValues
	 * (fonctionne comme une HashMap)
	 * 
	 * @param fiche
	 * @return code d'erreur
	 */
	public long insertSheet(Sheet sheet) {
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.SHEET_ID, sheet.getId());
		values.put(DataFieldDictionary.SHEET_NAME, sheet.getName());
		values.put(DataFieldDictionary.SHEET_IMAGE, sheet.getImage());
		values.put(DataFieldDictionary.SHEET_FAVORITE, sheet.getFavorite());
		values.put(DataFieldDictionary.SHEET_STATUS, sheet.getStatus());
		values.put(DataFieldDictionary.SHEET_NBCLICKS, sheet.getNbClicks());
		values.put(DataFieldDictionary.SHEET_NBCLICKS_DELTA, sheet
				.getNbClicksDelta());
		values.put(DataFieldDictionary.SHEET_DATE, sheet.getDate());
		values.put(DataFieldDictionary.SHEET_AUTHOR_ID, sheet.getAuthorId());
		values.put(DataFieldDictionary.SHEET_CATEGORY_ID, sheet
				.getCategory_id());
		values.put(DataFieldDictionary.SHEET_NOTE, sheet.getNote());

		return mDatabase.insert(SHEET_TABLE, null, values);

	}

	/**
	 * Insertion d'une taxonomy dans la base. Utilisation d'un ContentValues
	 * (fonctionne comme une HashMap)
	 * 
	 * @param tax
	 * @return code d'erreur
	 */
	public long insertTaxonomy(Taxonomy tax) {
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.TAX_TERM, tax.getTermId());
		values.put(DataFieldDictionary.TAX_SHEET, tax.getSheetId());

		return mDatabase.insert(TAXONOMY_TABLE, null, values);
	}

	/**
	 * Insertion d'un terme dans la base. Utilisation d'un ContentValues
	 * (fonctionne comme une HashMap)
	 * 
	 * @param term
	 * @return code d'erreur
	 */
	public long insertTerm(Term term) {
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.TERM_ID, term.getId());
		values.put(DataFieldDictionary.TERM_SPACE_ID, term.getSpaceId());
		values.put(DataFieldDictionary.TERM_NAME, term.getName());

		return mDatabase.insert(TERM_TABLE, null, values);
	}

	/**
	 * Mise a jour d'une categorie. Consiste a en creer une et a ecraser
	 * l'ancienne.
	 * 
	 * @param id
	 *            ID de la categorie
	 * @param category
	 * @return code d'erreur
	 */
	public int updateCategory(Category category) {
		int id = category.getId();
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.CAT_NAME, category.getName());
		values.put(DataFieldDictionary.CAT_IMAGE, category.getImage());
		values.put(DataFieldDictionary.CAT_PARENT, category.getParentId());
		values.put(DataFieldDictionary.CAT_RANK, category.getRank());

		return mDatabase.update(CATEGORY_TABLE, values,
				DataFieldDictionary.CAT_ID + " = " + id, null);

	}

	/**
	 * Mise a jour d'une fiche. Consiste a en creer une et a ecraser l'ancienne.
	 * 
	 * @param id
	 *            ID de la fiche
	 * @param categorie
	 * @return code d'erreur
	 */
	public int updateSheet(Sheet sheet) {
		ContentValues values = new ContentValues();
		int id = sheet.getId();
		values.put(DataFieldDictionary.SHEET_NAME, sheet.getName());
		values.put(DataFieldDictionary.SHEET_IMAGE, sheet.getImage());
		values.put(DataFieldDictionary.SHEET_FAVORITE, sheet.getFavorite());
		values.put(DataFieldDictionary.SHEET_STATUS, sheet.getStatus());
		values.put(DataFieldDictionary.SHEET_NBCLICKS, sheet.getNbClicks());
		values.put(DataFieldDictionary.SHEET_NBCLICKS_DELTA, sheet
				.getNbClicksDelta());
		values.put(DataFieldDictionary.SHEET_DATE, sheet.getDate());
		values.put(DataFieldDictionary.SHEET_CATEGORY_ID, sheet
				.getCategory_id());
		values.put(DataFieldDictionary.SHEET_AUTHOR_ID, sheet.getAuthorId());
		values.put(DataFieldDictionary.SHEET_NOTE, sheet.getNote());

		return mDatabase.update(SHEET_TABLE, values,
				DataFieldDictionary.SHEET_ID + " = " + id, null);
	}

	/**
	 * Incrementation du nombre de click sur une fiche
	 * 
	 * @param id
	 * @param sheet
	 * @return
	 */
	public int updateNbClicksSheet(Sheet sheet) {
		int id = sheet.getId();
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.SHEET_NBCLICKS_DELTA, sheet
				.getNbClicksDelta());
		values.put(DataFieldDictionary.SHEET_NBCLICKS, sheet.getNbClicks());

		return mDatabase.update(SHEET_TABLE, values,
				DataFieldDictionary.SHEET_ID + " = " + id, null);

	}

	/**
	 * Suppression d'une categorie avec son ID
	 * 
	 * @param id
	 * @return
	 */
	public int removeCategorieWithID(int id) {
		return mDatabase.delete(CATEGORY_TABLE, DataFieldDictionary.CAT_ID
				+ " = " + id, null);
	}

	/**
	 * Suppression de toutes les categories
	 * 
	 * @return
	 */
	public int removeAllCategories() {
		return mDatabase.delete(CATEGORY_TABLE, ALL_ROWS, null);
	}

	/**
	 * Suppression d'une fiche avec son ID
	 * 
	 * @param id
	 * @return
	 */
	public int removeSheetWithID(int id) {
		return mDatabase.delete(SHEET_TABLE, DataFieldDictionary.SHEET_ID
				+ " = " + id, null);
	}

	/**
	 * Suppression de tous les fiches
	 * 
	 * @return
	 */
	public int removeAllSheets() {
		return mDatabase.delete(SHEET_TABLE, ALL_ROWS, null);
	}

	/**
	 * Suppression de tous les fiches
	 * 
	 * @return
	 */
	public int removeAllTaxonomys() {
		return mDatabase.delete(TAXONOMY_TABLE, ALL_ROWS, null);
	}

	/**
	 * Suppression des taxonomys correspondant  un
	 * @param sheetId
	 * @return
	 */
	public int removeTaxonomyEntries(int sheetId) {
		return mDatabase.delete(TAXONOMY_TABLE, DataFieldDictionary.TAX_SHEET
				+ " = " + sheetId, null);
	}
	
	/**
	 * Suppression de tous les termes
	 * 
	 * @return
	 */
	public int removeAllTerms() {
		return mDatabase.delete(TERM_TABLE, ALL_ROWS, null);
	}

	

	
	/**
	 * Suppression de toutes les donnees
	 */
	public void removeAllDatas() {
		removeAllCategories();
		removeAllSheets();
		removeAllTaxonomys();
		removeAllTerms();
	}

	/**
	 * Recuperation de toutes les fiches
	 * 
	 * @return {@link ArrayList}
	 */
	public List<Category> getCategories() {
		if (needRefresh) {
			Cursor c = mDatabase.query(CATEGORY_TABLE, new String[] {
					DataFieldDictionary.CAT_ID, 
					DataFieldDictionary.CAT_NAME,
					DataFieldDictionary.CAT_IMAGE,
					DataFieldDictionary.CAT_PARENT,
					DataFieldDictionary.CAT_RANK }, null, null, null, null,
					null, null);
			categories = cursorToCategories(c);
		}
		return categories;
	}

	/**
	 * Recuperation d'une categorie avec son ID
	 * 
	 * @param id
	 * @return {@link Category}
	 */
	public Category getCategoryWithId(int id) {
	
		Category result = null;
		for (Category category : getCategories()) {
			if (category.getId() == id) {
				result = category;
			}
		}
		return result;
	}

	/**
	 * Recuperation des categories filles avec l'ID de la categorie mere
	 * 
	 * @param id
	 * @return {@link ArrayList}
	 */
	public List<Category> getChildCategories(int id) {

		List<Category> result = new LinkedList<Category>();
		for (Category category : getCategories()) {
			if (category.getParentId() == id && category.getId() != id) {
				result.add(category);
			}
		}
		Collections.sort(result, Collections.reverseOrder());
		return result;

	}

	/**
	 * Recuperation d'une fiche avec son ID
	 * 
	 * @param id
	 * @return {@link Sheet}
	 */
	public Sheet getSheetWithId(int id) {
		/*
		 * Cursor c = mDatabase.query(SHEET_TABLE, null, SHEET_ID + " = " + id,
		 * null, null, null, null); return cursorToSheet(c);
		 */
		Sheet result = null;
		for (Sheet sheet : getSheets()) {
			if (sheet.getId() == id) {
				result = sheet;
			}
		}
		return result;
	}

	/**
	 * Recuperation d'une fiche avec son image
	 * 
	 * @param id
	 * @return {@link Sheet}
	 */
	public Sheet getSheetWithImage(String image) {
	
		Sheet result = null;
		for (Sheet sheet : getSheets()) {
			if (sheet.getImage().equals(image)) {
				result = sheet;
			}
		}
		return result;

	}

	/**
	 * Recuperation de l'ID de la categorie en fonction de son image
	 * 
	 * @param imagePath
	 * @return
	 */
	public int getCategoryIdByImagePath(String imagePath) {

		Category result = null;
		for (Category category : getCategories()) {
			if (category.getImage().equals(imagePath)) {
				result = category;
			}
		}
		return result.getId();

	}

	/**
	 * Recuperation des fiches selon leur categorie. Les fiches sont classes
	 * 
	 * @param id
	 * @return {@link ArrayList}
	 */
	public List<Sheet> getSheetsByCategoryId(int id) {
	
		List<Sheet> result = new LinkedList<Sheet>();
		for (Sheet sheet : getSheets()) {
			if (sheet.getCategory_id() == id && sheet.getStatus() == 1) {
				result.add(sheet);
			}
		}
		Collections.sort(result, Collections.reverseOrder());
		return result;
	}

	/**
	 * Recuperation de toutes les fiches
	 * 
	 * @return {@link ArrayList}
	 */
	public List<Sheet> getSheets() {
		if (needRefresh) {
			Cursor c = mDatabase.query(SHEET_TABLE, null, null, null, null,
					null, null);
			sheets = cursorToSheets(c);
		}
		return sheets;
	}

	/**
	 * Recuperation des fiches favorites
	 * 
	 * @return {@link ArrayList}
	 */
	protected List<Sheet> getFavorites() {
	
		List<Sheet> result = new LinkedList<Sheet>();
		for (Sheet sheet : getSheets()) {
			if (sheet.getFavorite() == 1 && sheet.getStatus() == 1) {
				result.add(sheet);
			}
		}
		return result;
	}

	/**
	 * Recuperation de toutes les taxonomy
	 * 
	 * @param id
	 * @return {@link Taxonomy}
	 */
	public List<Taxonomy> getTaxonomys() {
		if (needRefresh) {
			Cursor c = mDatabase.query(TAXONOMY_TABLE, null, null, null, null,
					null, null);
			taxonomies = cursorToTaxonomys(c);
		}
		return taxonomies;
	}

	/**
	 * Recuperation de toutes les taxonomy
	 * 
	 * @param id
	 * @return {@link Taxonomy}
	 */
	public List<Term> getTerms() {
		if (needRefresh) {
			Cursor c = mDatabase.query(TERM_TABLE, null, null, null, null,
					null, null);
			terms = cursorToTerms(c);
		}
		return terms;
	}

	/**
	 * Cette methode permet de convertir un cursor en une categorie
	 * 
	 * @param c
	 * @return {@link Category}
	 */
	protected Category cursorToCategorie(Cursor c) {
		if (c.getCount() == 0)
			return null;
		c.moveToFirst();
		Category categorie = new Category(
				c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_ID)), 
				c.getString(c.getColumnIndex(DataFieldDictionary.CAT_NAME)), 
				c.getString(c.getColumnIndex(DataFieldDictionary.CAT_IMAGE)), 
				c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_PARENT)), 
				c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_RANK)));
		c.close();
		return categorie;
	}

	/**
	 * Cette methode permet de convertir un cursor en une fiche
	 * 
	 * @param c
	 * @return {@link Sheet}
	 */
	protected Sheet cursorToSheet(Cursor c) {
		if (c.getCount() == 0)
			return null;
		c.moveToFirst();

		Sheet sheet = createSheetFromCursor(c);
		c.close();
		return sheet;
	}

	/**
	 * Create a sheet from cursor
	 * 
	 * @param c
	 * @return {@link Sheet}
	 */
	/*
	 * Sheet(int id, String name, String description, String image, int
	 * favorite,int status, int nbClic, String date, int space_id, int
	 * category_id, int author_id, int note) {
	 */
	protected Sheet createSheetFromCursor(Cursor c) {
		Sheet sheet = new Sheet(
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_ID)),
				c.getString(c.getColumnIndex(DataFieldDictionary.SHEET_NAME)),
				c.getString(c.getColumnIndex(DataFieldDictionary.SHEET_IMAGE)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_FAVORITE)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_STATUS)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_NBCLICKS)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_NBCLICKS_DELTA)),
				c.getString(c.getColumnIndex(DataFieldDictionary.SHEET_DATE)),
				c.getString(c.getColumnIndex(DataFieldDictionary.SHEET_LAST_MODIF)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_CATEGORY_ID)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_AUTHOR_ID)),
				c.getInt(c.getColumnIndex(DataFieldDictionary.SHEET_NOTE)));
		return sheet;
	}

	/**
	 * Permet de convertir un cursor en ensemble de fiches
	 * 
	 * @param c
	 * @return {@link ArrayList}
	 */
	protected ArrayList<Sheet> cursorToSheets(Cursor c) {
		if (c.getCount() == 0)
			return new ArrayList<Sheet>();
		ArrayList<Sheet> alSheet = new ArrayList<Sheet>(c.getCount());
		c.moveToFirst();
		Sheet sheet;
		do {
			sheet = createSheetFromCursor(c);
			alSheet.add(sheet);
		} while (c.moveToNext());
		c.close();
		return alSheet;
	}

	/**
	 * Permet de convertir un cursor en ensemble de categories
	 * 
	 * @param c
	 * @return {@link ArrayList}
	 */
	protected ArrayList<Category> cursorToCategories(Cursor c) {
		if (c.getCount() == 0)
			return new ArrayList<Category>();
		ArrayList<Category> alCategory = new ArrayList<Category>(c.getCount());
		c.moveToFirst();
		do {
			Category categorie = new Category(
					c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_ID)),
					c.getString(c.getColumnIndex(DataFieldDictionary.CAT_NAME)),
					c.getString(c.getColumnIndex(DataFieldDictionary.CAT_IMAGE)),
					c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_PARENT)),
					c.getInt(c.getColumnIndex(DataFieldDictionary.CAT_RANK)));
			alCategory.add(categorie);
		} while (c.moveToNext());
		c.close();
		return alCategory;
	}

	/**
	 * Permet de convertir un cursor en term
	 * 
	 * @param c
	 * @return {@link Taxonomy}
	 */
	protected List<Term> cursorToTerms(Cursor c) {
		if (c.getCount() == 0)
			return new ArrayList<Term>();
		ArrayList<Term> terms = new ArrayList<Term>(c.getCount());
		c.moveToFirst();
		do {
			Term term = new Term(
					c.getInt(c.getColumnIndex(DataFieldDictionary.TERM_ID)), 
					c.getInt(c.getColumnIndex(DataFieldDictionary.TERM_SPACE_ID)), 
					c.getString(c.getColumnIndex(DataFieldDictionary.TERM_NAME)));
			terms.add(term);
		} while (c.moveToNext());
		c.close();
		return terms;
	}

	/**
	 * Permet de convertir un cursor en taxonomy
	 * 
	 * @param c
	 * @return {@link Taxonomy}
	 */
	protected List<Taxonomy> cursorToTaxonomys(Cursor c) {
		if (c.getCount() == 0)
			return new ArrayList<Taxonomy>();
		ArrayList<Taxonomy> taxonomys = new ArrayList<Taxonomy>(c.getCount());
		c.moveToFirst();
		do {
			Taxonomy taxonomy = new Taxonomy( 
					c.getInt(c.getColumnIndex(DataFieldDictionary.TAX_TERM)), 
					c.getInt(c.getColumnIndex(DataFieldDictionary.TAX_SHEET)));
			taxonomys.add(taxonomy);
		} while (c.moveToNext());
		c.close();
		return taxonomys;
	}

	/**
	 * Permet de convertir un cursor en ensemble d'entier representant les ID
	 * des fiches
	 * 
	 * @param c
	 * @return {@link ArrayList}
	 */
	protected List<Integer> cursorToSheetsByCategoryId(Cursor c) {
		if (c.getCount() == 0)
			return new ArrayList<Integer>();
		ArrayList<Integer> listeId = new ArrayList<Integer>(c.getCount());
		c.moveToFirst();
		do {
			Integer i = new Integer(c.getInt(0));
			listeId.add(i);
		} while (c.moveToNext());
		c.close();
		return listeId;
	}

	protected List<Category> getMainCategories() {

		List<Category> result = new LinkedList<Category>();
		for (Category category : getCategories()) {
			if (category.getParentId() == 0) {
				result.add(category);
			}
		}
		return result;
	}

	/**
	 * Pour recupere les categories dans la base de données
	 * 
	 * @return linkedHashMap
	 */
	public Map<String, Integer> getImageForCategories() {
		Map<String, Integer> result = new LinkedHashMap<String, Integer>();
		List<Category> categories = this.getMainCategories();
		for (Category s : categories) {
			result.put(s.getImage(), s.getId());
		}
		return result;
	}

	/**
	 * Pour recupere les favoris dans la base de données
	 * 
	 * @return String[]
	 */
	public List<String> getImageForFavorites() {
		List<Sheet> sheets = this.getFavorites();
		List<String> images = new LinkedList<String>();
		for (Sheet s : sheets) {
			images.add(s.getImage());
		}
		return images;

	}

	/**
	 * Return a map containing couples containing SheetId and number of clicks
	 * Only couple with non-zero number of clicks are returned
	 */
	public Map<Integer, Integer> getNbClicksDeltaForAllSheets() {
		Map<Integer, Integer> map = new HashMap<Integer, Integer>();
		Cursor c = mDatabase.query(SHEET_TABLE, new String[] { DataFieldDictionary.SHEET_ID,
				DataFieldDictionary.SHEET_NBCLICKS_DELTA }, DataFieldDictionary.SHEET_NBCLICKS_DELTA + "<> 0", null,
				null, null, null);
		if (c.getCount() != 0) {
			c.moveToFirst();
			do {
				Integer id = new Integer(c.getInt(0));
				Integer nbclicksDelta = new Integer(c.getInt(1));
				map.put(id, nbclicksDelta);
			} while (c.moveToNext());
		}
		c.close();
		return map;
	}

	/**
	 * Reset delta click counter of all sheets
	 */
	public int resetNbClicksDelta() {
		ContentValues values = new ContentValues();
		values.put(DataFieldDictionary.SHEET_NBCLICKS_DELTA, 0);
		return mDatabase.update(SHEET_TABLE, values, null, null);
	}

	
}