/**
 * Project: Max'Com: an oral communication software solution that provides
 * pre-recorded sound <-> daylife image / word associations. Autistic people
 * or persons with communication problems are to be the main users. The
 * content is sorted by categories that helps users to express themselves
 * according to the context.
 * File name: DatabaseHelper.java
 * Description: this class is used to update the database.
 * 
 * @author Telecom Lille1 on 2010-11-23
 * @copyright 2010 Max'Com team. All rights reserved.
 * @version $Id: code-gpl-license.txt,v 1.2 2004/05/04 13:19:30 garry Exp $
 * 
 * @see The GNU Public License (GPL) version 3 or higher
 *
 * 
 * Max'Com is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package maxcom.mobile;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;

import maxcom.mobile.MaxCom.ErrorType;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.net.Uri;
import android.util.Log;

public class DatabaseSynchronizer {

	public static final String tag = DatabaseSynchronizer.class.getName();

	protected static SheetCmdImageUtil sheetCmdImageUtil = new SheetCmdImageUtil();
	protected static CategoryCmdImageUtil categoryCmdImageUtil = new CategoryCmdImageUtil();

	protected Context mContext;

	protected int mNbUpdate;
	protected int mNbUpdateErrors;
	protected int mNbInsert;
	protected int mNbInsertErrors;
	protected int mNbDelete;
	protected int mNbDeleteErrors;

	protected List<String> mMessageErrors;

	protected DatabaseCmdFactory cmdFactory;

	protected static final int OK = 200;

	public DatabaseSynchronizer(Context context) {
		mContext = context;
		mMessageErrors = new LinkedList<String>();
		cmdFactory = new DatabaseCmdFactory();
		reset();
	}

	/**
	 * Reset action counters
	 */
	protected void reset() {
		mMessageErrors.clear();

		mNbUpdate = 0;
		mNbInsert = 0;
		mNbDelete = 0;
		mNbUpdateErrors = 0;
		mNbInsertErrors = 0;
		mNbDeleteErrors = 0;
	}

	/**
	 * Return number of database updates for the last synchro
	 */
	public int getNbUpdate() {
		return mNbUpdate;
	}

	/**
	 * Return number of database inserts for the last synchro
	 */
	public int getNbInsert() {
		return mNbInsert;
	}

	/**
	 * Return number of database deletes for the last synchro
	 */
	public int getNbDelete() {
		return mNbDelete;
	}

	/**
	 * Return number of errors caused by data updates for the last synchro
	 */
	public int getNbUpdateErrors() {
		return mNbUpdateErrors;
	}

	/**
	 * Return number of errors caused by data inserts for the last synchro
	 */
	public int getNbInsertErrors() {
		return mNbInsertErrors;
	}

	/**
	 * Return number of errors caused by data delete for the last synchro
	 */
	public int getNbDeleteErrors() {
		return mNbDeleteErrors;
	}

	public List<String> getMessageErrors() {
		return mMessageErrors;
	}

	/**
	 * Indicates if the last synchro has no action
	 * 
	 * @return
	 */
	public boolean isEmptySynchro() {
		return (mNbInsert + mNbDelete + mNbUpdate) == 0;
	}

	/**
	 * Indicates if the last synchro has errors
	 * 
	 * @return
	 */
	public boolean isSynchroWithErrors() {
		return (mNbInsertErrors + mNbDeleteErrors + mNbUpdateErrors) > 0;
	}

	/**
	 * Update the local database from remote database actions
	 */
	public void update() throws IOException, ClientProtocolException {
		reset();
		String result = retrieveDbActions();
		if (!result.equals("")) {
			processDbActions(result);
		}
	}

	/**
	 * Return Command part of synchro URL
	 * 
	 * @return
	 */
	public String getCmdPart() {
		return "/show_queries";
		// old cmd part return "/get_action";
	}

	/**
	 * Compute an URL segment for synchronized with the server
	 */
	protected String getSynchroURL() {
		Uri serverUri = ConfigurationData.getServerUri();
		String synchroPart = MaxCom.getSynchroPart();
		String login = ConfigurationData.getLogin();
		String password = ConfigurationData.getEncryptedLoginPassword();
		Date lastUpdate = ConfigurationData.getDateLastUpdate();
		String dateLastUpdate = new SimpleDateFormat("yyyy-MM-dd/HH:mm:ss",
				Locale.US).format(lastUpdate);
		String action = getCmdPart() + "/" + login + "/" + password + "/"
				+ dateLastUpdate;
		return Uri.withAppendedPath(serverUri, synchroPart + action).toString();
	}

	/**
	 * Retrieve database actions from remote site
	 * 
	 * @return
	 * @throws Exception
	 */
	protected String retrieveDbActions() throws IOException {
		String result = "";
		InputStream is = null;
		ConfigurationData.load(mContext);

		// retrieve config data to perform the synchronisation with the correct
		// user and date

		try {
			// creation and transmission of the corresponding HTTP request

			HttpClient httpClient = MaxCom.createHttpClient();

			String synchroURL = getSynchroURL();

			HttpGet httpGet = new HttpGet(synchroURL);
			if (BuildConfig.DEBUG)
				Log.i(tag, "Download from:" + httpGet.getURI().toString());

			HttpResponse response = httpClient.execute(httpGet);

			if (BuildConfig.DEBUG)
				Log.i(tag, "Response status:" + response.getStatusLine());

			// Http response handling and payload extraction
			if (response.getStatusLine().getStatusCode() == OK) {
				HttpEntity entity = response.getEntity();
				if (entity != null) {
					is = entity.getContent();
					// convert response to string
					BufferedReader reader = new BufferedReader(
							new InputStreamReader(is, "UTF-8"), 8);
					StringBuilder sb = new StringBuilder();
					String line = null;
					while ((line = reader.readLine()) != null) {
						sb.append(line + "\n");
					}
					is.close();
					result = sb.toString();
				}
			} else {
				MaxCom.logError(ErrorType.Communication_Error, response
						.getStatusLine().toString());
			}
		} catch (IOException e) {
			MaxCom.logError(ErrorType.Connection_Error, e);
			throw e;
		} catch (Exception e) {
			MaxCom.logError(ErrorType.Internal_Error, e);
		}
		return result;
	}

	/**
	 * Transform database action into command objects and evaluate the commands
	 * 
	 * @param result
	 */
	protected void processDbActions(String result) {

		try {
			// parse json data
			JSONArray jArray = new JSONArray(result);
			if (jArray.length() > 0) {
				DatabaseHelper dbHelper = MaxCom.getDatabaseHelper();

				for (int i = 0; i < jArray.length(); i++) {
					if (!dbHelper.isOpened())
						dbHelper.open();
					DatabaseCmd dbCmd = null;
					try {
						JSONObject jsonCmd = jArray.getJSONObject(i);
						dbCmd = cmdFactory.createCmd(jsonCmd);
						if (dbCmd != null) {
							dbCmd.evaluate(dbHelper);
							if (dbCmd.isDeleteCmd())
								mNbDelete++;
							else if (dbCmd.isUpdateCmd())
								mNbUpdate++;
							else if (dbCmd.isInsertCmd())
								mNbInsert++;
							if (BuildConfig.DEBUG)
								Log.i(tag, "Evaluated command:" + jsonCmd);
						} else {
							if (BuildConfig.DEBUG)
								Log.i(tag, "Ignored invalid command:" + jsonCmd);
						}
					} catch (Exception e) {
						MaxCom.logError(ErrorType.Database_Error, e);
						mMessageErrors.add(e.getMessage());
						if (dbCmd != null) {
							if (dbCmd.isDeleteCmd())
								mNbDeleteErrors++;
							else if (dbCmd.isUpdateCmd())
								mNbUpdateErrors++;
							else if (dbCmd.isInsertCmd())
								mNbInsertErrors++;
						}
					}
				}

				dbHelper.close();

				// write the last update date in the configuration file
				ConfigurationData.setLastUpdate(new Date());
				ConfigurationData.save(mContext);
			}
		} catch (Exception e) {
			Log.e(tag, e.toString());
		}
	}

	/**
	 * This class factorizes common methods for translating JSON Object to
	 * entries in database
	 * 
	 * @author gva
	 * 
	 */
	protected abstract class JSONCmdUtil {

		protected final static String JSON_ID = "id";
		protected final static String JSON_SPACE_ID = "space_id";
		protected final static String JSON_DATE = "date";
		protected final static String JSON_TIME = "time";

		protected final static String JSON_REQ_TYPE = "req_type";
		protected final static String JSON_REQ_DATAS = "req_datas";
		protected final static String JSON_ITEM_TYPE = "item_type";
		protected final static String JSON_WHERE = "where";

		protected boolean hasField(JSONObject json, String field) {
			return json.has(field);
		}

		protected int getIntField(JSONObject json, String field) {
			int res = DataFieldDictionary.VOID_INT;
			try {
				res = json.getInt(field);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		protected String getStringField(JSONObject json, String field) {
			String res = DataFieldDictionary.VOID_STR;
			try {
				res = json.getString(field);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		protected boolean getBooleanField(JSONObject json, String field) {
			boolean res = false;
			try {
				int val = json.getInt(field);
				res = (val == 0) ? false : true;
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		protected JSONArray getArrayField(JSONObject json, String field) {
			JSONArray res = DataFieldDictionary.VOID_JSONARRAY;
			try {
				res = json.getJSONArray(field);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		protected JSONObject getObjectField(JSONObject json, String field) {
			JSONObject res = DataFieldDictionary.VOID_JSONOBJ;
			try {
				res = json.getJSONObject(field);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}
	}

	/**
	 * This class factorizes some methods for all command subclasses
	 * 
	 * @author gva
	 * 
	 */
	protected abstract class DatabaseCmd extends JSONCmdUtil {
		protected JSONObject jsonData;

		public DatabaseCmd(JSONObject json) {
			jsonData = json;
		}

		public abstract void evaluate(DatabaseHelper dbHelper);

		public boolean isDeleteCmd() {
			return false;
		}

		public boolean isUpdateCmd() {
			return false;
		}

		public boolean isInsertCmd() {
			return false;
		}

		public int getID() {
			return getIntField(jsonData, JSON_ID);
		}

		public int getSpaceID() {
			return getIntField(jsonData, JSON_SPACE_ID);
		}

		public String getDate() {
			return getStringField(jsonData, JSON_DATE);
		}

		public String getTime() {
			return getStringField(jsonData, JSON_TIME);
		}

		public String getItemType() {
			return getStringField(jsonData, JSON_ITEM_TYPE);
		}

		public String getReqType() {
			return getStringField(jsonData, JSON_REQ_TYPE);
		}

		public JSONObject getReqData() {
			return getObjectField(jsonData, JSON_REQ_DATAS);
		}

		public String getWherePart1() {
			JSONArray array = getArrayField(jsonData, JSON_WHERE);
			String res = null;
			try {
				res = array.getString(0);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		public int getWherePart2() {
			JSONArray array = getArrayField(jsonData, JSON_WHERE);
			int res = -1;
			try {
				res = array.getInt(1);
			} catch (JSONException ex) {
				ex.printStackTrace();
			}
			return res;
		}

		public String getSheetName() {
			return getStringField(getReqData(), DataFieldDictionary.SHEET_NAME);
		}

		public boolean hasSheetName() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_NAME);
		}

		public int getSheetStatus() {
			return getIntField(getReqData(), DataFieldDictionary.SHEET_STATUS);
		}

		public boolean hasSheetStatus() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_STATUS);
		}

		public int getSheetFavorite() {
			return getIntField(getReqData(), DataFieldDictionary.SHEET_FAVORITE);
		}

		public boolean hasSheetFavorite() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_FAVORITE);
		}

		public int getSheetNote() {
			return getIntField(getReqData(), DataFieldDictionary.SHEET_NOTE);
		}

		public boolean hasSheetNote() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_NOTE);
		}

		public int getSheetCategoryId() {
			return getIntField(getReqData(),
					DataFieldDictionary.SHEET_CATEGORY_ID);
		}

		public boolean hasSheetCategoryId() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_CATEGORY_ID);
		}

		public String getSheetImage() {
			return getStringField(getReqData(), DataFieldDictionary.SHEET_IMAGE);
		}

		public boolean hasSheetImage() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_IMAGE);
		}

		public int getSheetAuthorId() {
			return getIntField(getReqData(),
					DataFieldDictionary.SHEET_AUTHOR_ID);
		}

		public boolean hasSheetAuthorId() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_AUTHOR_ID);
		}

		public String getSheetDate() {
			return getStringField(getReqData(), DataFieldDictionary.SHEET_DATE);
		}

		public boolean hasSheetDate() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_DATE);
		}

		public String getSheetLastModif() {
			return getStringField(getReqData(),
					DataFieldDictionary.SHEET_LAST_MODIF);
		}

		public boolean hasSheetLastModif() {
			return hasField(getReqData(), DataFieldDictionary.SHEET_LAST_MODIF);
		}

		public String getCategoryName() {
			return getStringField(getReqData(), DataFieldDictionary.CAT_NAME);
		}

		public boolean hasCategoryName() {
			return hasField(getReqData(), DataFieldDictionary.CAT_NAME);
		}

		public String getCategoryImage() {
			return getStringField(getReqData(), DataFieldDictionary.CAT_IMAGE);
		}

		public boolean hasCategoryImage() {
			return hasField(getReqData(), DataFieldDictionary.CAT_IMAGE);
		}

		public int getCategoryParentId() {
			return getIntField(getReqData(), DataFieldDictionary.CAT_PARENT);
		}

		public boolean hasCategoryParentId() {
			return hasField(getReqData(), DataFieldDictionary.CAT_PARENT);
		}

		public int getCategoryRank() {
			return getIntField(getReqData(), DataFieldDictionary.CAT_RANK);
		}

		public boolean hasCategoryRank() {
			return hasField(getReqData(), DataFieldDictionary.CAT_RANK);
		}

		public JSONArray getTaxonomyTerms() {
			return getArrayField(jsonData, JSON_REQ_DATAS);
		}

		public JSONArray getTermName() {
			return getArrayField(jsonData, JSON_REQ_DATAS);
		}

		protected void logEntity(Object obj) {
			String reqType = getReqType();
			if (reqType != null)
				if (BuildConfig.DEBUG)
					Log.i(tag, reqType + ": " + obj);
		}
	}

	// DeleteCmd sub-hierarchy
	protected abstract class DeleteCmd extends DatabaseCmd {

		public DeleteCmd(JSONObject json) {
			super(json);
		}

		@Override
		public boolean isDeleteCmd() {
			return true;
		}

		protected void addLoadedImage(List<String> images, String imageName) {
			if (!imageName.equals("warning_category.png")
					|| !imageName.equals("warning_sheet.png")) {
				images.add(imageName);
			}
		}

		protected void removeImageFiles(List<String> images) {
			// remove the image files
			for (String image : images) {
				if (!image.equals("")) {
					File file = new File(MaxCom.getImagesDirectory(), image);
					file.delete();
				}
			}
		}
	}

	protected class SheetDeleteCmd extends DeleteCmd {
		public SheetDeleteCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {
			int sheetId = getWherePart2();
			List<String> images = new LinkedList<String>();

			Sheet sheet = dbHelper.getSheetWithId(sheetId);
			if (sheet != null) {
				addLoadedImage(images, sheet.getImage());
				removeImageFiles(images);

				dbHelper.removeSheetWithID(sheetId);
			}
		}
	}

	protected class CategoryDeleteCmd extends DeleteCmd {
		public CategoryDeleteCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {
			int categoryId = getWherePart2();
			List<String> images = new LinkedList<String>();

			Category category = dbHelper.getCategoryWithId(categoryId);
			if (category != null) {
				addLoadedImage(images, category.getImage());

				List<Sheet> sheets = dbHelper.getSheetsByCategoryId(categoryId);
				for (Sheet sheet : sheets) {
					addLoadedImage(images, sheet.getImage());
				}
				removeImageFiles(images);

				dbHelper.removeCategorieWithID(categoryId);
			}
		}

	}

	// UpdateCmd sub-hierarchy
	protected abstract class InsertCmd extends DatabaseCmd {
		public InsertCmd(JSONObject json) {
			super(json);
		}

		@Override
		public boolean isInsertCmd() {
			return true;
		}
	}

	protected class SheetInsertCmd extends InsertCmd {

		public SheetInsertCmd(JSONObject json) {
			super(json);

		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {
			int sheetId = getWherePart2();
			String image = getSheetImage();

			if (!MaxCom.isVoidImage(image)) {
				boolean downloaded = sheetCmdImageUtil.downloadImage(image);
				if (!downloaded) image = "";
			}
			
			Sheet sheet = new Sheet(sheetId, getSheetName(), getSheetImage(),
					getSheetFavorite(), getSheetStatus(), 0, 0, getSheetDate(),
					getSheetLastModif(), getSheetCategoryId(),
					getSheetAuthorId(), getSheetNote());
			dbHelper.insertSheet(sheet);
			logEntity(sheet);
		}

	}

	protected class CategoryInsertCmd extends InsertCmd {

		public CategoryInsertCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {
			int categoryId = getWherePart2();
			String image = getCategoryImage();

			if (!MaxCom.isVoidImage(image)) {
				boolean downloaded = categoryCmdImageUtil.downloadImage(image);
				if (!downloaded) image = "";  
			}
			
			Category category = new Category(categoryId, getCategoryName(),
					getCategoryImage(), getCategoryParentId(),
					getCategoryRank());
			dbHelper.insertCategory(category);
			logEntity(category);			
		}
	}

	protected class TermInsertCmd extends InsertCmd {
		public TermInsertCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {

			int termId = getWherePart2();

			JSONArray datas = getTermName();
			int space_id = getSpaceID();
			String name = "none";
			try {
				name = datas.getString(0);
			} catch (JSONException e) {
				e.printStackTrace();
			}

			Term term = new Term(termId, space_id, name);
			dbHelper.insertTerm(term);
			logEntity(term);

		}
	}

	// UpdateCmd sub-hierarchy
	protected abstract class UpdateCmd extends DatabaseCmd {
		public UpdateCmd(JSONObject json) {
			super(json);
		}

		@Override
		public boolean isUpdateCmd() {
			return true;
		}
	}

	protected class CategoryUpdateCmd extends UpdateCmd {

		public CategoryUpdateCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {

			int categoryId = getWherePart2();
			Category category = dbHelper.getCategoryWithId(categoryId);

			if (category != null) {
				String oldImage = null;
				String newImage = null;

				if (hasCategoryName())
					category.setName(getCategoryName());
				if (hasCategoryImage()) {
					oldImage = category.getImage();
					newImage = getCategoryImage();
					category.setImage(newImage);
				}
				if (hasCategoryParentId())
					category.setParentId(getCategoryParentId());
				if (hasCategoryRank())
					category.setRank(getCategoryRank());

				dbHelper.updateCategory(category);
				logEntity(category);

				if (!MaxCom.isVoidImage(newImage)) {
					boolean downloaded = categoryCmdImageUtil
							.downloadImage(newImage);
					if (downloaded) {
						categoryCmdImageUtil.removeImageFromDirectory(oldImage);
					}
				}
			}
		}

	}

	protected class SheetUpdateCmd extends UpdateCmd {

		public SheetUpdateCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {

			int sheetId = getWherePart2();

			Sheet sheet = dbHelper.getSheetWithId(sheetId);
			if (sheet != null) {
				String oldImage = null;
				String newImage = null;
				if (hasSheetImage()) {
					oldImage = sheet.getImage();
					newImage = getSheetImage();
				}

				sheet.setImage(newImage);
				if (hasSheetDate())
					sheet.setDate(getSheetDate());
				if (hasSheetCategoryId())
					sheet.setCategoryId(getSheetCategoryId());
				if (hasSheetFavorite())
					sheet.setFavorite(getSheetFavorite());
				if (hasSheetLastModif())
					sheet.setLastModif(getSheetLastModif());
				if (hasSheetName())
					sheet.setName(getSheetName());
				if (hasSheetStatus())
					sheet.setStatus(getSheetStatus());
				if (hasSheetNote())
					sheet.setNote(getSheetNote());
				if (hasSheetAuthorId())
					sheet.setAuthorId(getSheetAuthorId());

				dbHelper.updateSheet(sheet);
				logEntity(sheet);

				if (!MaxCom.isVoidImage(newImage)) {
					boolean downloaded = sheetCmdImageUtil
							.downloadImage(newImage);
					if (downloaded) {
						sheetCmdImageUtil.removeImageFromDirectory(oldImage);
					}
				}
			}
		}
	}

	protected class TaxonomyUpdateCmd extends UpdateCmd {
		public TaxonomyUpdateCmd(JSONObject json) {
			super(json);
		}

		@Override
		public void evaluate(DatabaseHelper dbHelper) {

			int sheetId = getWherePart2();
			JSONArray datas = getTaxonomyTerms();

			// before insertion, we delete existing taxonomy entries
			// for the selected sheet
			dbHelper.removeTaxonomyEntries(sheetId);

			int length = datas.length();
			for (int i = 0; i < length; i++) {
				try {
					int termId = datas.getInt(i);
					Taxonomy taxonomy = new Taxonomy(termId, sheetId);
					dbHelper.insertTaxonomy(taxonomy);
					logEntity(taxonomy);

				} catch (JSONException e) {
					e.printStackTrace();
				}
			}
		}
	}

	protected class DatabaseCmdFactory extends JSONCmdUtil {

		final String tag = DatabaseCmdFactory.class.getName();

		protected final static String INSERT_CMD = "INSERT";
		protected final static String UPDATE_CMD = "UPDATE";
		protected final static String DELETE_CMD = "DELETE";

		protected final static String CATEGORY_ITEM_TYPE = "category";
		protected final static String SHEET_ITEM_TYPE = "sheet";
		protected final static String TAXONOMY_ITEM_TYE = "taxonomy";
		protected final static String TERM_ITEM_TYPE = "term";

		public DatabaseCmd createCmd(JSONObject json) {
			DatabaseCmd newCmd = null;
			// Category cmd
			if (isItemTypeEqual(json, CATEGORY_ITEM_TYPE)) {
				if (isReqTypeEqual(json, INSERT_CMD)) {
					newCmd = new CategoryInsertCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Category insert from:" + json);
				}
				if (isReqTypeEqual(json, UPDATE_CMD)) {
					newCmd = new CategoryUpdateCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Category update from:" + json);
				}
				if (isReqTypeEqual(json, DELETE_CMD)) {
					newCmd = new CategoryDeleteCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Category delete from:" + json);
				}
			}

			if (isItemTypeEqual(json, SHEET_ITEM_TYPE)) {
				if (isReqTypeEqual(json, INSERT_CMD)) {
					newCmd = new SheetInsertCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Sheet insert from:" + json);
				}
				if (isReqTypeEqual(json, UPDATE_CMD)) {
					newCmd = new SheetUpdateCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Sheet update from:" + json);
				}
				if (isReqTypeEqual(json, DELETE_CMD)) {
					newCmd = new SheetDeleteCmd(json);
					if (BuildConfig.DEBUG)
						Log.i(tag, "Sheet delete from:" + json);
				}
			}

			if (isItemTypeEqual(json, TAXONOMY_ITEM_TYE)) {
				if (isReqTypeEqual(json, UPDATE_CMD)) {
					newCmd = new TaxonomyUpdateCmd(json);
					Log.i(tag, "Taxonomy update from:" + json);
				}
			}

			if (isItemTypeEqual(json, TERM_ITEM_TYPE)) {
				if (isReqTypeEqual(json, INSERT_CMD)) {
					newCmd = new TermInsertCmd(json);
					Log.i(tag, "Term insert from:" + json);
				}
			}
			return newCmd;
		}

		protected boolean isItemTypeEqual(JSONObject json, String type) {
			String itemType = getStringField(json, JSON_ITEM_TYPE);
			if ((itemType != null) && (itemType.equals(type)))
				return true;
			else
				return false;
		}

		protected boolean isReqTypeEqual(JSONObject json, String type) {
			String reqType = getStringField(json, JSON_REQ_TYPE);
			if ((reqType != null) && (reqType.equals(type)))
				return true;
			else
				return false;
		}
	}

	/**
	 * Send usage statistics to server
	 * 
	 * @throws Exception
	 */

	void sendStatsUsage() throws Exception {

		// We retrieve delta of clicks for all sheets
		DatabaseHelper dbHelper = MaxCom.getDatabaseHelper();
		dbHelper.open();
		Map<Integer, Integer> clicksMap = dbHelper
				.getNbClicksDeltaForAllSheets();
		dbHelper.close();

		if (!clicksMap.isEmpty()) {

			JSONObject jsonStats = new JSONObject();
			jsonStats.put("id", MaxCom.getAndroidID());
			jsonStats.put("model", MaxCom.getDeviceModel());

			// Map is transformed into a JSON object
			JSONObject jsonClicks = new JSONObject();
			for (Entry<Integer, Integer> entry : clicksMap.entrySet()) {
				jsonClicks.put(entry.getKey().toString(), entry.getValue()
						.toString());
			}
			jsonStats.put("clicks", jsonClicks);
			if (BuildConfig.DEBUG)
				Log.i(tag, "Stats:" + jsonStats.toString());

			String statsURL = getStatsURL();

			// We sent a POST request containing the number of clicks
			HttpClient httpClient = new DefaultHttpClient();
			HttpPost httpPost = new HttpPost(statsURL);

			StringEntity clicStatsEntity = new StringEntity(
					jsonStats.toString());
			clicStatsEntity.setContentType("application/json");
			httpPost.setEntity(clicStatsEntity);

			if (BuildConfig.DEBUG)
				Log.i(tag, "Sending stats data to:"
						+ httpPost.getURI().toString());

			HttpResponse response = httpClient.execute(httpPost);
			if (BuildConfig.DEBUG)
				Log.i(tag, "Response status:" + response.getStatusLine());

			if (response.getStatusLine().getStatusCode() != OK) {
				MaxCom.logError(ErrorType.Communication_Error, ""
						+ response.getStatusLine().getStatusCode());
			} else {
				// we reset delta of clicks if sending is successful
				dbHelper.open();
				dbHelper.resetNbClicksDelta();
				dbHelper.close();
			}
		}
	}

	/**
	 * Compute an URL segment for sending statistics to server
	 */
	protected String getStatsURL() {
		Uri serverUri = ConfigurationData.getServerUri();
		String loggingPart = "index.php/logging";
		String login = ConfigurationData.getLogin();
		String password = ConfigurationData.getEncryptedLoginPassword();
		String action = "/process_stats/" + login + "/" + password;

		return Uri.withAppendedPath(serverUri, loggingPart + action).toString();
	}

}
