/**
 * Project: Max'Com: an oral communication software solution that provides
 * pre-recorded sound <-> daylife image / word associations. Autistic people
 * or persons with communication problems are to be the main users. The
 * content is sorted by categories that helps users to express themselves
 * according to the context.
 * File name: MaxCom.java
 * Description: This class is used for shared objects and global utility 
 * functions.
 * 
 * @author Telecom Lille1 on 2010-11-23
 * @copyright 2010 Max'Com team. All rights reserved.
 * @version $Id: code-gpl-license.txt,v 1.2 2004/05/04 13:19:30 garry Exp $
 * 
 * @see The GNU Public License (GPL) version 3 or higher
 *
 * 
 * Max'Com is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 * Application Class for shared objects and global utility functions
 * @author Telecom Lille1
 *
 */
package maxcom.mobile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.FileEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.app.Application;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.provider.Settings;
import android.util.Log;

public class MaxCom extends Application {

	protected static final String tag = MaxCom.class.getName();

	protected static DatabaseHelper dbHelper;
	protected static DatabaseSynchronizer dbSynchronizer;

	protected static SheetTermClassifier termClassifier;

	protected final static String picturesDirectoryName = "pictures";
	protected static File imageDirectory;

	protected static Logger logger;
	protected final static String loggerName = "maxcom";
	protected final static String logFileName = "maxcom.log";

	protected static final int OK = 200;

	protected static final int connectionTimeoutMillis = 300;
	protected static final int socketTimeoutMillis = 300;

	protected static String androidID;
	protected static int sdkVersion;
	protected static String deviceModel;

	protected static MaxCom appInstance;

	@Override
	public void onCreate() {
		super.onCreate();
		dbHelper = new DatabaseHelper(this);
		dbSynchronizer = new DatabaseSynchronizer(this);
		termClassifier = new SheetTermClassifier();

		imageDirectory = getDir(picturesDirectoryName, Context.MODE_PRIVATE);

		initDeviceInfo();
		loadConfig();
		sendLogFile();
		initLogger();

		appInstance = this;
	}

	protected void initDeviceInfo() {
		androidID = Settings.Secure.getString(getContentResolver(),
				Settings.Secure.ANDROID_ID);
		sdkVersion = Build.VERSION.SDK_INT;
		deviceModel = Build.MODEL;
	}

	/**
	 * Load the configuration data
	 */
	protected void loadConfig() {
		if (ConfigurationData.isConfigCreated(getApplicationContext())) {
			ConfigurationData.load(getApplicationContext());
		}
	}

	/**
	 * Send the last log file to the server if possible and delete it in case of
	 * success
	 */
	protected void sendLogFile() {
		try {
			if (ConfigurationData.isLoaded()) {
				File file = new File(getFilesDir(), logFileName);
				if (file.exists()) {
					FileEntity fileEntity = new FileEntity(file, "text/plain");

					// creation and transmission of the corresponding HTTP
					// request
					HttpClient httpClient = createHttpClient();

					String logURL = getLogURL();
					HttpPost httpPost = new HttpPost(logURL);
					httpPost.setEntity(fileEntity);

					HttpResponse response = httpClient.execute(httpPost);
					// Log file is delete if reception is OK
					if (response.getStatusLine().getStatusCode() == OK) {
						file.delete();
					}
				}
			}
		} catch (Exception e) {
			// no way to log this error since logger is not ready
			e.printStackTrace();
		}
	}

	/**
	 * Compute an URL segment for sending logs to server
	 */
	protected String getLogURL() {
		Uri serverUri = ConfigurationData.getServerUri();
		String synchroPart = "index.php/logging/add_record/";
		String userPart = ConfigurationData.getLogin() + "/"
				+ ConfigurationData.getEncryptedLoginPassword();

		return Uri.withAppendedPath(serverUri, synchroPart + userPart)
				.toString();
	}

	/**
	 * Create the logger to log errors
	 */
	protected void initLogger() {
		logger = Logger.getLogger(loggerName);
		try {
			File file = new File(getFilesDir(), logFileName);
			if (!file.exists()) {
				file.createNewFile();
				FileOutputStream ostream = this.openFileOutput(logFileName,
						Context.MODE_APPEND);
				PrintStream pstream = new PrintStream(ostream);
				String lineSep = System.getProperty("line.separator");
				String header = "{ \"id\": " + "\"" + MaxCom.getAndroidID()
						+ "\"" + "," + "\"model\": " + "\""
						+ MaxCom.getDeviceModel() + "\"" + ","
						+ "\"version\" :" + "\"" + MaxCom.getSdkVersion()
						+ "\"" + "}" + lineSep;
				pstream.print(header);
				ostream.close();
			}
			FileHandler fileHandler = new FileHandler(file.getCanonicalPath(),
					true);
			fileHandler.setFormatter(new LogFormatter());
			logger.addHandler(fileHandler);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create a HTTP Client with appropriate time-out parameters
	 * 
	 * @return
	 */
	public static HttpClient createHttpClient() {
		HttpClient httpClient = new DefaultHttpClient();
		HttpParams params = httpClient.getParams();
		HttpConnectionParams.setConnectionTimeout(params,
				connectionTimeoutMillis);
		HttpConnectionParams.setSoTimeout(params, socketTimeoutMillis);
		return httpClient;
	}

	/**
	 * Return the device id
	 * 
	 * @return
	 */
	public static String getAndroidID() {
		return androidID;
	}

	/**
	 * Return the sdk version used by this device
	 * 
	 * @return
	 */

	public static int getSdkVersion() {
		return sdkVersion;
	}

	/**
	 * Return the device model
	 * 
	 * @return
	 */
	public static String getDeviceModel() {
		return deviceModel;
	}

	/**
	 * Return the object for managing the database
	 * 
	 * @return
	 */
	public static DatabaseHelper getDatabaseHelper() {
		return dbHelper;
	}

	/**
	 * Return the synchro segment
	 * 
	 * @return
	 */
	public static String getSynchroPart() {
		return "index.php/synchro";
	}

	/**
	 * Return the object for performing the synchronization
	 * 
	 * @return
	 */
	public static DatabaseSynchronizer getDatabaseSynchonizer() {
		return dbSynchronizer;
	}

	public static SheetTermClassifier getTermClassifier() {
		return termClassifier;
	}

	/**
	 * Return the directory for storing the downloaded pictures (Lazy creation)
	 * 
	 * @return
	 */
	public static File getImagesDirectory() {
		return imageDirectory;
	}

	/**
	 * Return the bitmap of a downloaded image
	 * 
	 * @param image
	 *            filename
	 * @return the bitmap or null if not exist
	 */

	public static Bitmap getImageBitmap(String name) {
		File file = null;
		InputStream is = null;
		Bitmap bitmap = null;
		try {
			// test if image name is valid before loading
			if (!isVoidImage(name)) {
				file = new File(MaxCom.getImagesDirectory(), name);
				is = new FileInputStream(file);
				bitmap = BitmapFactory.decodeStream(is);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (bitmap == null)
				bitmap = getNoImageBitmap();
			try {
				is.close();
			} catch (Exception e) {
			}
		}
		return bitmap;
	}

	/**
	 * Return a boolean to indicate if the image is void
	 * 
	 * @param image
	 *            name
	 * @return the bitmap or null if not exist
	 */
	public static boolean isVoidImage(String image) {
		return (image == null || image.equals("") || image.equals("0"));
	}

	protected static Bitmap getNoImageBitmap() {
		return BitmapFactory.decodeResource(appInstance.getResources(),
				R.drawable.nopicture);
	}

	/*
	 * Enum for main error types
	 */
	public enum ErrorType {
		Update_Error, Connection_Error, Communication_Error, Database_Error, Configuration_Error, File_Error, Internal_Error
	};

	/**
	 * Log an error with a type and a message
	 */
	public static void logError(ErrorType type, String msg) {
		if (BuildConfig.DEBUG)
			Log.i(tag, type.toString());
		logger.log(Level.INFO, type.toString(), msg);
	}

	/**
	 * Log an error with a type and a related exception
	 */
	public static void logError(ErrorType type, Exception ex) {
		if (BuildConfig.DEBUG)
			Log.i(tag, type.toString());
		logger.log(Level.SEVERE, type.toString(), ex);
	}

}
