package maxcom.mobile;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import android.util.Log;

/**
 * Class for classifying sheets according to their terms
 * 
 * @author Telecom Lille1, Isen Lille
 * 
 */
public class SheetTermClassifier {

	final static int MAX_SIMILAR_SHEETS = 6;

	protected static final String tag = SheetTermClassifier.class.getName();

	protected Set<Sheet> sheets;
	protected Set<Term> terms;
	protected Map<Integer, Set<String>> termsMap;
	protected Map<Integer, Set<Sheet>> sheetsMap;

	public SheetTermClassifier() {
		sheets = new HashSet<Sheet>();
		terms = new HashSet<Term>();
		termsMap = new HashMap<Integer, Set<String>>();
		sheetsMap = new HashMap<Integer, Set<Sheet>>();
	}

	/**
	 * Fill the lists with data and compute sheet similarity
	 * 
	 * @param ls
	 * @param lt
	 * @param taxonomys
	 */
	public void fillWithData(List<Sheet> ls, List<Term> lt,
			List<Taxonomy> taxonomys) {
		// clear alls data first
		sheets.clear();
		terms.clear();
		termsMap.clear();
		sheetsMap.clear();

		sheets.addAll(ls);
		terms.addAll(lt);
		fillTermsMap(taxonomys);
		computeSimilarity();
	}

	/**
	 * Return the term with a given Id
	 * 
	 * @param id
	 * @return
	 */
	protected Term findTermWithId(int id) {
		Term term = null;
		for (Term t : terms) {
			if (t.getId() == id) {
				term = t;
				break;
			}
		}
		return term;
	}

	/**
	 * Return the sheet with a given id
	 * 
	 * @param id
	 * @return
	 */
	protected Sheet findSheetWithId(int id) {
		Sheet sheet = null;
		for (Sheet sh : sheets) {
			if (sh.getId() == id) {
				sheet = sh;
				break;
			}
		}
		return sheet;
	}

	/**
	 * Fill a map with couples containing sheet as key and their terms as value
	 * 
	 * @param taxonomys
	 */
	protected void fillTermsMap(List<Taxonomy> taxonomys) {
		for (Taxonomy t : taxonomys) {
			Term term = findTermWithId(t.getTermId());
			Sheet sheet = findSheetWithId(t.getSheetId());
			if ((term != null) && (sheet != null)) {
				Set<String> mappedTerms = termsMap.get(sheet.getId());
				if (mappedTerms == null) {
					mappedTerms = new HashSet<String>();
				}
				mappedTerms.add(term.getName());
				termsMap.put(sheet.getId(), mappedTerms);
			}
		}
	}

	/**
	 * Memorize in a map the order of similar sheets for each sheet
	 */
	protected void computeSimilarity() {

		for (Sheet sheet : sheets) {

			Set<Sheet> similarSheets = getSimilarityOrderFor(sheet);
			if (BuildConfig.DEBUG)
				Log.i(tag, "Computed similarity for sheet " + sheet + ":"
						+ similarSheets);
			sheetsMap.put(sheet.getId(), similarSheets);
		}
	}

	/**
	 * Compute the ordered list of similar sheets for a given sheet
	 * 
	 * @param sheet
	 * @return
	 */
	protected Set<Sheet> getSimilarityOrderFor(Sheet sheet) {
		SheetSimilarityComparator comparator = new SheetSimilarityComparator(
				sheet);
		Set<Sheet> otherSheets = new HashSet<Sheet>();
		// we remove sheet from the list
		for (Sheet s : sheets) {
			double similarity = comparator.getSimilarity(sheet, s);
			if (s.getId() != sheet.getId() && similarity != 0) {
				otherSheets.add(s);
			}
		}

		List<Sheet> lst = new LinkedList<Sheet>(otherSheets);
		Collections.sort(lst, comparator);
		int size = lst.size();
		if (size > MAX_SIMILAR_SHEETS)
			size = MAX_SIMILAR_SHEETS;
		return new HashSet<Sheet>(lst.subList(0, size));
	}

	/**
	 * Return the recorded list of similar sheets for a given sheet
	 * 
	 * @param sheet
	 * @return
	 */
	public Set<Sheet> getSimilarSheets(Sheet sheet) {
		Set<Sheet> result = sheetsMap.get(sheet.getId());
		if (result == null)
			result = new HashSet<Sheet>();
		return result;
	}

	/**
	 * Inner class that compare similarity between two sheets relatively to a
	 * given sheet using Jacquard distance
	 * 
	 */
	public class SheetSimilarityComparator implements Comparator<Sheet> {
		protected Sheet sheet;

		SheetSimilarityComparator(Sheet s) {
			sheet = s;
		}

		/**
		 * Compare two sheets relatively to a given sheet
		 */
		public int compare(Sheet sheet1, Sheet sheet2) {
			double sim1 = getSimilarity(sheet1, sheet);
			double sim2 = getSimilarity(sheet2, sheet);
			if (sim1 < sim2)
				return 1;
			else if (sim1 > sim2)
				return -1;
			else
				return 0;
		}

		/**
		 * Compute Jacquard distance between two sheets
		 * 
		 * @param sheet1
		 * @param sheet2
		 * @return
		 */
		public double getSimilarity(Sheet sheet1, Sheet sheet2) {

			// jacquard similarity
			int intersectItems = 0;
			Set<String> termSheet1 = termsMap.get(sheet1.getId());
			Set<String> termSheet2 = termsMap.get(sheet2.getId());
			if (BuildConfig.DEBUG)
				Log.i(tag, "Compare:" + termSheet1 + "-" + termSheet2);
			if (termSheet1 != null && termSheet2 != null) {
				for (String term1 : termSheet1) {
					for (String term2 : termSheet2) {
						// Find the same item
						if (term1.equals(term2)) {
							intersectItems++;
						}
					}
				}

				int unionItems = 0;
				Set<String> set = new HashSet<String>();
				set.addAll(termSheet1);
				set.addAll(termSheet2);
				unionItems = set.size();
				return ((double) intersectItems) / ((double) unionItems);
			} else
				return 0;
		}
	}
}
